google.load("visualization", "1", {packages: ["corechart", "bar"]});

$(document).ready(function () {
    $("#data-tables-wrapper").resizable({
        handles: 'w'
    });

    $('.chart-modal').dialog({
        autoOpen: false,
        height: $(window).height() * .4,
        width: $(window).width() * .6,
        modal: true
    });
    //hide z,w,v axis by default
    document.getElementById("zAxis").style.display = "none";
    document.getElementById("wAxis").style.display = "none";
    document.getElementById("vAxis").style.display = "none";

    //hide y axis only when Pie chart type is selected
    $('.chart-modal').find('.chart-type').change(function () {
        if ($(this).val() == 'Pie') {
            $('.yAxis-opts').fadeOut(200, function () {
                console.log($(this).val());
                if ($(this).val() == null) {
                    $('.yAxis-opts option').attr('selected', false);
                    $('.yAxis-opts option:first').attr('selected', true);
                }
            });
        } else {
            $('.yAxis-opts').fadeIn(200);
        }
        if ($(this).val() == 'Bubble') { // Aaron added fade out z, w, vAxis if not Bubble
            $('.zAxis-opts').fadeIn(200);
            $('.wAxis-opts').fadeIn(200);
            $('.vAxis-opts').fadeIn(200);
        } else {
            $('.zAxis-opts').fadeOut(200, function () {
                console.log($(this).val());
                if ($(this).val() == null) {
                    $('.zAxis-opts option').attr('selected', false);
                    $('.zAxis-opts option:first').attr('selected', true);
                }
            });
            $('.wAxis-opts').fadeOut(200, function () {
                console.log($(this).val());
                if ($(this).val() == null) {
                    $('.wAxis-opts option').attr('selected', false);
                    $('.wAxis-opts option:first').attr('selected', true);
                }
            });
            $('.vAxis-opts').fadeOut(200, function () {
                console.log($(this).val());
                if ($(this).val() == null) {
                    $('.vAxis-opts option').attr('selected', false);
                    $('.vAxis-opts option:first').attr('selected', true);
                }
            });
        }
    });
    //show bucket size input only when Histogram chart type is selected
    document.getElementById("num-buckets").style.display = "none";
    $('.chart-modal').find('.chart-type').change(function () {
        if ($(this).val() != 'Histogram') {
            document.getElementById("num-buckets").style.display = "none";
            $('.bucketSize').fadeOut(200);
        } else {
            document.getElementById("num-buckets").style.display = "block";
            $('.bucketSize').fadeIn(200);
        }
    });
});

// function to create charts
function createChart() {
    var modal = $(".chart-modal");
    var dataType = modal.find('.modal-data-type').val();
    var chartType = modal.find('.chart-type').val();
    var xVal = modal.find('.xAxis-opts').val();
    var yVal = modal.find('.yAxis-opts').val();
    var zVal = modal.find('.zAxis-opts').val(); // Aaron added zVal
    var wVal = modal.find('.wAxis-opts').val(); // Aaron added wVal
    var vVal = modal.find('.vAxis-opts').val(); // Aaron added vVal
    var numBuckets = document.getElementById("num-buckets").value;

    // validate
    var errorText = [];
    if (chartType == '' || chartType == null) {
        errorText.push("<p>Chart Type is required.</p>");
    }
    if (xVal == '' || xVal == null) {
        errorText.push("<p>X Axis is required.</p>");
    }
    if (yVal == '' || yVal == null) {
        errorText.push("<p>Y Axis is required.</p>");
    }
    if (chartType == "Bubble") {
        if (zVal == '' || zVal == null) {
            errorText.push("<p>Z Axis is required.</p>");
        }
    }
    if (errorText.length > 0) {
        modal.find('.errorText').html(errorText).fadeIn();
        return false;
    } else {
        modal.find('.errorText').html('').fadeOut();
    }
    var tableData = dataType == "query" ? queryChartData : nodeChartData;
    var chartData = [];

    // get x/y values from chart data
    $.each(tableData, function (key, val) {
        //chartData.push([val[xVal], parseFloat(val[yVal])]);
        chartData.push([val[xVal], val[yVal], val[zVal], val[wVal], val[vVal]]); // Aaron added zVal, wVal and vVal
    });

    var chart, data, options;
    var wrapper = document.getElementById('chart-wrapper');

    if (chartType == "Area") {
        chart = new google.visualization.AreaChart(wrapper);
        data = new google.visualization.DataTable();
        data.addColumn('string', xVal);
        data.addColumn('number', yVal);
        $.each(chartData, function (key, arr) {
            data.addRow([arr[0].toString(), parseFloat(arr[1])]);
        });
        options = ({
            legend: {position: 'none'},
            animation: {startup: true, duration: 1000}
        });
        chart.draw(data, options);
    }

    else if (chartType == "Bar") {
        chart = new google.charts.Bar(wrapper);
        data = new google.visualization.DataTable();
        data.addColumn('string', xVal);
        data.addColumn('number', yVal);
        $.each(chartData, function (key, arr) {
            data.addRow([arr[0].toString(), parseFloat(arr[1])]);
        });
        options = ({
            legend: {position: 'none'},
            animation: {startup: true, duration: 1000},
            bars: "horizontal"
        });
        chart.draw(data, google.charts.Bar.convertOptions(options));

    }

    else if (chartType == "Bubble") { // Aaron added Bubble ChartType
        chart = new google.visualization.BubbleChart(wrapper);
        data = new google.visualization.DataTable();
        data.addColumn('string', zVal);
        data.addColumn('number', xVal);
        data.addColumn('number', yVal);

        // truthy if statements to determine add w and v values only if they exist
        if(wVal && !vVal)
            data.addColumn('string', wVal);
        if(vVal && !wVal) {
            // data.addColumn('string', 'test');
            data.addColumn('string', "Country");
            data.addColumn('number', vVal);
        }
        if(wVal && vVal){
            data.addColumn('string', wVal);
            data.addColumn('number', vVal);
        }


        $.each(chartData, function (key, arr) {
            if (wVal && vVal)
                data.addRow([arr[2].toString(), parseFloat(arr[0]), parseFloat(arr[1]), arr[3].toString(), parseFloat(arr[4])]);
            else if (wVal && !vVal)
                data.addRow([arr[2].toString(), parseFloat(arr[0]), parseFloat(arr[1]), arr[3].toString()]);
            else if (vVal && !wVal)
                data.addRow([arr[2].toString(), parseFloat(arr[0]), parseFloat(arr[1]), arr[2].toString(), parseFloat(arr[4])]);
            else
                data.addRow([arr[2].toString(), parseFloat(arr[0]), parseFloat(arr[1])]);
        });

        options = ({
            bubble: {textStyle: {fontSize: 11}},
            legend: {position: 'none'},
            animation: {startup: true, duration: 1000}
        });
        chart.draw(data, options);
    }

    else if (chartType == "Column") {
        chart = new google.charts.Bar(wrapper);
        data = new google.visualization.DataTable();
        data.addColumn('string', xVal);
        data.addColumn('number', yVal);
        $.each(chartData, function (key, arr) {
            data.addRow([arr[0].toString(), parseFloat(arr[1])]);
        });
        options = ({
            legend: {position: 'none'},
            animation: {startup: true, duration: 1000},
            bars: "vertical"
        });
        chart.draw(data, google.charts.Bar.convertOptions(options));
    }

    else if (chartType == "Histogram") {
        chart = new google.visualization.Histogram(wrapper);
        data = new google.visualization.DataTable();
        data.addColumn('string', xVal);
        data.addColumn('number', yVal);
        $.each(chartData, function (key, arr) {
            data.addRow([arr[0].toString(), parseFloat(arr[1])]);
        });
        options = ({
            legend: {position: 'none'},
            animation: {startup: true, duration: 1000},
            histogram: {bucketSize: numBuckets}
        });
        chart.draw(data, options);
    }

    else if (chartType == "Line") {
        chart = new google.visualization.LineChart(wrapper);
        data = new google.visualization.DataTable();
        data.addColumn('string', xVal);
        data.addColumn('number', yVal);
        $.each(chartData, function (key, arr) {
            data.addRow([arr[0].toString(), parseFloat(arr[1])]);
        });
        options = ({
            legend: {position: 'none'},
            animation: {startup: true, duration: 1000}
        });
        chart.draw(data, options);
    }

    else if (chartType == "Pie") {
        chart = new google.visualization.PieChart(wrapper);
        data = new google.visualization.DataTable();
        data.addColumn('string', xVal);
        data.addColumn('number', 'Count');
        // get aggregated counts
        var groupedCount = {};
        $.each(chartData, function (key, arr) {
            // does group exist?
            if (groupedCount[arr[0]] > 0) {
                groupedCount[arr[0]]++;
            } else {
                groupedCount[arr[0]] = 1;
            }
        });


        $.each(groupedCount, function (key, val) {
            data.addRow([key + " (" + val + ")", val]);
        });
        options = ({
            animation: {startup: true, duration: 1000},
            is3D: true
        });
        chart.draw(data, options);
    }

    else if (chartType == "Scatter") {
        chart = new google.visualization.ScatterChart(wrapper);
        data = new google.visualization.DataTable();
        data.addColumn('string', xVal);
        data.addColumn('number', yVal);
        $.each(chartData, function (key, arr) {
            data.addRow([arr[0].toString(), parseFloat(arr[1])]);
        });
        options = ({
            legend: {position: 'none'}
            ,
            animation: {startup: true, duration: 1000}
        });
        chart.draw(data, options);
    }

}